@@ -5,7 +5,7 @@ SporeStack API request/response models
 """
 
 
-from typing import List, Optional
+from typing import Dict, List, Optional
 
 from pydantic import BaseModel
 
@@ -51,7 +51,6 @@ class ServerLaunch:
         """Token to draw from when launching the server."""
         quote: bool = False
         """Don't launch, get a quote on how much it would cost"""
-        affiliate_token: Optional[str] = None
         hostname: str = ""
         """Hostname to refer to your server by."""
         autorenew: bool = False
@@ -65,17 +64,11 @@ class ServerLaunch:
         """Deprecated, not needed when paying with token. Only used for quote."""
         expiration: int
         machine_id: str
-        network_interfaces: List[NetworkInterface] = []
-        """Deprecated, use ipv4/ipv6 from ServerInfo instead."""
         created_at: int = 0
-        region: Optional[str] = None
-        """Deprecated, use ServerInfo instead."""
         created: bool = False
         paid: bool = False
         """Deprecated, not needed when paying with token."""
         warning: Optional[str] = None
-        flavor: str = ""
-        """Deprecated, use ServerInfo instead."""
 
 
 class ServerTopup:
@@ -85,15 +78,11 @@ class ServerTopup:
     class Request(BaseModel):
         days: int
         token: str
-        quote: bool = False
-        affiliate_token: Optional[str] = None
 
     class Response(BaseModel):
         machine_id: str
-        payment: Payment
-        """Deprecated, not needed when paying with token."""
         expiration: int
-        paid: bool = False
+        paid: bool = True
         """Deprecated, not needed when paying with token."""
         warning: Optional[str] = None
 
@@ -173,7 +162,7 @@ class Flavors:
     method = "GET"
 
     class Response(BaseModel):
-        flavors: dict[str, Flavor]
+        flavors: Dict[str, Flavor]
 
 
 class OperatingSystems:
